-- ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
-- Written by Kimberly L. Tripp, SQLskills.com
-- All Rights Reserved.
-- 
-- For more scripts, sample code and additional training resources,
-- go to http://www.SQLskills.com
--	Immerse yourself in SQL Server
--
-- Disclaimer - Thoroughly test this script, execute at your own risk.
-- ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

USE Credit
go

-- Get the count just to know... 
SELECT count(c.*) 
FROM dbo.charge AS c
go

SELECT MAX(c.charge_amt) 
FROM dbo.charge AS c
go

-- Turn on Statistics IO so that you can see the number of I/Os needed to read
-- to get this data.
SET STATISTICS IO ON
go

SELECT c.* 
FROM dbo.charge AS c
WHERE c.charge_amt > 6000
go
-- 0 ROWS... yet many I/Os because it performed a Table Scan

ALTER TABLE dbo.charge
ADD CONSTRAINT ChargeChargeAmtCKCnstIsTrusted
	CHECK (charge_amt < 5010)
go

SELECT c.* 
FROM dbo.charge AS c
WHERE c.charge_amt > 6000
go
-- 0 ROWS... and 0 I/Os because the constraint knew there was no data?

ALTER TABLE dbo.charge
	DROP CONSTRAINT ChargeChargeAmtCKCnstIsTrusted
go

-- To prove the point - how about creating the constraint with NOCHECK
ALTER TABLE dbo.charge
	WITH NOCHECK -- This is evil :)
ADD CONSTRAINT ChargeChargeAmtCKCnstIsNOTTrusted
	CHECK (charge_amt < 5010)
go

SET STATISTICS IO ON
go
SELECT c.* 
FROM dbo.charge AS c
WHERE c.charge_amt > 6000
go
-- 0 ROWS... back to many I/Os because NOCHECK created a NON-trusted constraint!

-- You can verify the constraints...
-- But - even if you use DBCC CHECKCONSTRAINTS and NO data fails verification, 
-- the data is NOT truly verified because the table is not locked during the DBCC
-- CHECKCONSTRAINTS command the same way it is when the constraint is added with
-- CHECK. So - the end result is that the optimizer will NOT trust the constraint.

-- To see if the constraint is trusted... 
SELECT OBJECTPROPERTY(object_id('ChargeChargeAmtCKCnstIsNOTTrusted'), 'CnstIsNOTTrusted') 
go
-- should be 1 because the constraint is NOT trusted...

-- To check the data you can use DBCC CHECKCONSTRAINTS
DBCC CHECKCONSTRAINTS('charge') -- all constraints on the table will be checked
DBCC CHECKCONSTRAINTS('ChargeChargeAmtCKCnstIsNOTTrusted') -- only THIS constraint will be checked
-- NOTE: Constraint names MUST be uniquely named within the database.
go
SELECT OBJECTPROPERTY(object_id('ChargeChargeAmtCKCnstIsNOTTrusted'), 'CnstIsNOTTrusted') 
go

-- Even if the data did not produce errors it's still NOT trusted because only the create 
-- with CHECK properly sets this bit (because of how it locks the table).
-- IF you absolutely want the data consistency issues of having a check constraint WITHOUT the 
-- "downtime" of having to do a table lock then you can add the constraint with NOCHECK (at 10am
-- on a Monday morning, for example) and then SCHEDULE an off hours DROP/re-CREATE of the 
-- constraint. This will ensure that the optimizer properly uses the constraint wherever it can!